package com.taais.biz.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.mybatisflex.core.query.QueryWrapper;
import com.taais.common.core.config.TaaisConfig;
import com.taais.common.core.core.domain.CommonResult;
import com.taais.common.core.core.page.PageResult;
import com.taais.common.core.utils.MapstructUtils;
import com.taais.common.core.utils.StringUtils;
import com.taais.common.core.utils.file.FileUploadUtils;
import com.taais.common.core.utils.file.FileUtils;
import com.taais.common.core.utils.file.UnPackedUtil;
import com.taais.common.excel.utils.ExcelUtil;
import com.taais.common.log.annotation.Log;
import com.taais.common.log.enums.BusinessType;
import com.taais.common.web.annotation.RepeatSubmit;
import com.taais.common.web.core.BaseController;
import com.taais.biz.domain.Data;
import com.taais.biz.domain.bo.DataBo;
import com.taais.biz.domain.vo.DataVo;
import com.taais.biz.service.IDataService;
import com.taais.system.config.ServerConfig;
import com.taais.system.domain.vo.SysOssUploadVo;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import net.lingala.zip4j.model.FileHeader;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;

/**
 * 数据管理Controller
 *
 * @author km
 * 2024-04-22
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/demo/data")
public class DataController extends BaseController {
    @Resource
    private IDataService dataService;
    @Autowired
    private ServerConfig serverConfig;

    /**
     * 查询数据管理列表
     */
    @SaCheckPermission("demo:data:list")
    @GetMapping("/list")
    public CommonResult<PageResult<DataVo>> list(DataBo dataBo) {

        return CommonResult.success(dataService.selectPage(dataBo));

    }

    /**
     * 导出数据管理列表
     */
    @SaCheckPermission("demo:data:export")
    @Log(title = "数据管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, DataBo dataBo) {
        List<DataVo> list = dataService.selectList(dataBo);
        ExcelUtil.exportExcel(list, "数据管理", DataVo.class, response);
    }

    /**
     * 获取数据管理详细信息
     */
    @SaCheckPermission("demo:data:query")
    @GetMapping(value = "/{id}")
    public CommonResult<DataVo> getInfo(@PathVariable Long id) {

        return CommonResult.success(dataService.selectById(id));
    }

    /**
     * 新增数据管理
     */
    @SaCheckPermission("demo:data:add")
    @Log(title = "数据管理", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody DataBo dataBo) {
        QueryWrapper query = dataService.query();
        query.eq(Data::getBatchNum, dataBo.getBatchNum());
        long count = dataService.getMapper().selectCountByQuery(query);
        if (count > 0) {
            return CommonResult.fail("检测出现重复批次号，请仔细检查后重新添加！");
        }
        boolean inserted = dataService.insert(dataBo);
        if (!inserted) {
            return CommonResult.fail("新增数据管理记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改数据管理
     */
    @SaCheckPermission("demo:data:edit")
    @Log(title = "数据管理", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody DataBo dataBo) {
        Boolean updated = dataService.update(dataBo);
        if (!updated) {
            return CommonResult.fail("修改数据管理记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除数据管理
     */
    @SaCheckPermission("demo:data:remove")
    @Log(title = "数据管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = dataService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除数据管理记录失败!");
        }
        return CommonResult.success();
    }

    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) {
        ExcelUtil.exportExcel(new ArrayList<>(), "用户数据", DataVo.class, response);
    }

    @PostMapping("/zip/upload")
    public CommonResult<SysOssUploadVo> uploadZipFile(@RequestParam("file") MultipartFile file, @RequestParam("excelFile") MultipartFile excelFile, String labeled) throws Exception {
        System.out.println(labeled);
        List<DataVo> dataVos = ExcelUtil.importExcel(excelFile.getInputStream(), DataVo.class);
        Map<String, Data> picName2Obj = new HashMap<>();
        //为插入数据库做准备
        for (DataVo dataVo : dataVos) {
            picName2Obj.put(dataVo.getName(), MapstructUtils.convert(dataVo, Data.class));
        }
        String originalFilename = file.getOriginalFilename();
        assert originalFilename != null;
        String suffix = StringUtils.substring(originalFilename, originalFilename.lastIndexOf("."), originalFilename.length());


        // 解压
        String basedir = TaaisConfig.getUploadPath();
        File zipOrRarTemp = FileUploadUtils.getAbsoluteFile(basedir, FileUploadUtils.extractFilename2(file));// 解压目录
        String destZip = zipOrRarTemp.getAbsolutePath();

        file.transferTo(Paths.get(destZip));
        String dest = zipOrRarTemp.getParent();

        List<File> extractedFileList = new ArrayList<File>();
        if (suffix.equals(".zip")) {
            List<FileHeader> fileheaders = UnPackedUtil.unPackZip(zipOrRarTemp, dest);
            //添加解压目录
            for (FileHeader fileHeader : fileheaders) {
                if (!fileHeader.isDirectory()) {
                    String fileHeaderName = fileHeader.getFileName();
                    String fileHeaderSuffix = StringUtils.substring(fileHeaderName, fileHeaderName.lastIndexOf("."), fileHeaderName.length());
                    if (!fileHeaderSuffix.equals(".jpg") && !fileHeaderSuffix.equals(".jpeg")
                            && !fileHeaderSuffix.equals(".png")) {
                        continue;
                    }
                    extractedFileList.add(new File(dest, fileHeader.getFileName()));
                }
            }


        } else if (suffix.equals(".rar")) {
            List<com.github.junrar.rarfile.FileHeader> fileheaders = UnPackedUtil.unPackRar(zipOrRarTemp, dest);


            //添加解压目录
            for (com.github.junrar.rarfile.FileHeader fileHeader : fileheaders) {

                if (!fileHeader.isDirectory()) {
                    String fileHeaderName = fileHeader.getFileName();
                    String fileHeaderSuffix = StringUtils.substring(fileHeaderName, fileHeaderName.lastIndexOf("."), fileHeaderName.length());
                    if (!fileHeaderSuffix.equals(".jpg")
                            && !fileHeaderSuffix.equals(".jpeg")
                            && !fileHeaderSuffix.equals(".png")
                            && !fileHeaderSuffix.equals(".txt")
                            && !fileHeaderSuffix.equals(".bmp")) {
                        continue;
                    }
                    extractedFileList.add(new File(dest, fileHeader.getFileName()));
                }
            }
        }
        //取出最大值
        Integer idMax = dataService.getIdMax();
        if (idMax == null) {
            idMax = 1;
        } else {
            idMax = idMax + 1;
        }
        List<Data> dataList = new ArrayList<>();
        for (File efile : extractedFileList) {
            String pathFileName = FileUploadUtils.getPathFileName(dest, efile.getName());
            Data mydata = picName2Obj.get(efile.getName());
            // 检查是否已标准抓住
            if (checkLabeled(pathFileName)) {
                mydata.setLabeled(Boolean.TRUE);
            } else {
                mydata.setLabeled(Boolean.FALSE);
            }
            mydata.setFormName(mydata.getName());
            String fileHeaderSuffix = StringUtils.substring(mydata.getName(), mydata.getName().lastIndexOf("."), mydata.getName().length());
            mydata.setName(idMax + fileHeaderSuffix);
            mydata.setUrl(pathFileName);
            dataList.add(mydata);
            idMax++;
        }

        if ("true".equals(labeled)) {
            List<Data> unmarkedData = dataList.stream()
                    .filter(data -> Boolean.FALSE.equals(data.getLabeled()))
                    .toList();

            // 计算已标注和未标注的数据数量
            long markedCount = dataList.stream()
                    .filter(data -> Boolean.TRUE.equals(data.getLabeled()))
                    .count();
            long unmarkedCount = unmarkedData.size();

            // 如果存在未标注的数据，则返回错误信息
            if (unmarkedCount > 0) {
                String format = String.format("错误: 已标注文件 %d 个，未标注文件 %d 个", markedCount, unmarkedCount);
                return CommonResult.fail(format);
            }
        }

        AtomicBoolean isTrue = new AtomicBoolean(false);
        dataList.forEach(data -> {
            QueryWrapper query = dataService.query();
            query.eq(Data::getBatchNum, data.getBatchNum());
            long count = dataService.getMapper().selectCountByQuery(query);
            if (count > 0) {
                isTrue.set(true);
            }
        });

        if (isTrue.get()) {
            return CommonResult.fail("文件检测出重复批次号，请仔细检查后重新导入！");
        }

        dataService.saveBatch(dataList);
        //删除压缩文件
        FileUtils.deleteFile(destZip);


        return CommonResult.success("数据集上传成功！");
    }

    // 检查有没有标注信息有的话把标注数据置为已标注
    private boolean checkLabeled(String pathFileName) {
        Path newPath = FileUtils.getTxtPath(pathFileName);

        // 检查替换后的文件是否存在
        return Files.exists(newPath);
    }

}
